/*
 * Copyright 2016-present Open Networking Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package org.onosproject.fpcagent.rest;

import com.fasterxml.jackson.databind.node.ObjectNode;
import org.apache.commons.lang.exception.ExceptionUtils;
import org.apache.http.HttpVersion;
import org.apache.http.client.ResponseHandler;
import org.apache.http.client.methods.CloseableHttpResponse;
import org.apache.http.client.methods.HttpGet;
import org.apache.http.impl.client.BasicResponseHandler;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.onosproject.rest.AbstractWebResource;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

import javax.ws.rs.Consumes;
import javax.ws.rs.POST;
import javax.ws.rs.Path;
import javax.ws.rs.core.MediaType;
import javax.ws.rs.core.Response;
import java.io.IOException;
import java.io.InputStream;

@Path("/")
public class FpcWebResource extends AbstractWebResource {
    private static final Logger log =
            LoggerFactory.getLogger(FpcWebResource.class);

    private ResponseHandler<String> handler = new BasicResponseHandler();

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/request")
    public Response getRequest(InputStream stream) {
        try {
            ObjectNode node = (ObjectNode) mapper().readTree(stream);

            log.info("Received {}", node.toString());

            CloseableHttpClient client = HttpClients.createDefault();
            HttpGet httpGet = new HttpGet(node.get("client-uri").asText());

            httpGet.addHeader("User-Agent", "ONOS FPC Agent");
            httpGet.addHeader("Charset", "utf-8");
            httpGet.addHeader("Content-type", "text/event-stream");
            httpGet.setProtocolVersion(HttpVersion.HTTP_1_1);

            CloseableHttpResponse httpResponse = client.execute(httpGet);

            String response = handler.handleResponse(httpResponse);
            log.info("Response {}", response);
        } catch (IOException e) {
            log.error(ExceptionUtils.getFullStackTrace(e));
        }

        return Response.ok().build();
    }

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/response")
    public Response getResponse(InputStream stream) {
        try {
            ObjectNode node = (ObjectNode) mapper().readTree(stream);

            log.info("Received {}", node.toString());
        } catch (IOException e) {
            log.error(ExceptionUtils.getFullStackTrace(e));
        }
        return Response.ok().build();
    }

    @POST
    @Consumes(MediaType.APPLICATION_JSON)
    @Path("/notification")
    public Response getNotification(InputStream stream) {
        try {
            ObjectNode node = (ObjectNode) mapper().readTree(stream);

            log.info("Received {}", node.toString());
        } catch (IOException e) {
            log.error(ExceptionUtils.getFullStackTrace(e));
        }
        return Response.ok().build();
    }
}

